/**
 * vim: set ts=4 :
 * =============================================================================
 * SourceMod Sample Extension
 * Copyright (C) 2004-2008 AlliedModders LLC.  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#include "extension.h"

SteamID g_Interface;
SMEXT_LINK(&g_Interface);

cell_t GetClientCSteamID(IPluginContext *pContext, const cell_t *params)
{
	int index = params[1];
	if (!index)
	{
		return 0;
	}

	edict_t *entity = engine->PEntityOfEntIndex(index);
	if (entity == NULL)
	{
		return 0;
	}

	const CSteamID *steamID = engine->GetClientSteamID(entity);
	if (steamID == NULL)
	{
		return 0;
	}

	char *buf;
	pContext->LocalToString(params[2], &buf);
#ifdef PLATFORM_LINUX
	snprintf(buf, static_cast<size_t>(params[3]), "%llu", steamID->ConvertToUint64());
#else
	snprintf(buf, static_cast<size_t>(params[3]), "%I64d", steamID->ConvertToUint64());
#endif

	return 1;
}

cell_t CSteamIDToSteamID(IPluginContext *pContext, const cell_t *params)
{
	char *input, *buf;
	pContext->LocalToString(params[1], &input);
	pContext->LocalToString(params[2], &buf);

	if (strlen(input) < 17)
	{
		return 0;
	}

#ifdef PLATFORM_LINUX
	uint64 u64SteamID = (uint64)strtoll(input, NULL, 0);
#else
	uint64 u64SteamID = _atoi64(input);
#endif
	
	CSteamID steamID = CSteamID(u64SteamID);
	if (steamID.GetEAccountType() > k_EAccountTypeMax || steamID.GetEUniverse() > k_EUniverseMax ||	steamID.GetEUniverse() < 0 || !steamID.IsValid())
	{
		return 0;
	}

#ifdef PLATFORM_LINUX
	uint32 u32AccountID = steamID.GetAccountID();
	snprintf(buf, static_cast<size_t>(params[3]), "STEAM_0:%u:%u", (u32AccountID % 2) ? 1 : 0, (int32)u32AccountID/2);
#else
	snprintf(buf, static_cast<size_t>(params[3]), "STEAM_0:%u:%u", (steamID.m_unAccountID % 2) ? 1 : 0, (int32)steamID.m_unAccountID/2);
#endif

	return 1;
}

cell_t SteamIDToCSteamID(IPluginContext *pContext, const cell_t *params)
{
	char *input, *buf;
	pContext->LocalToString(params[1], &input);
	pContext->LocalToString(params[2], &buf);

	if (strlen(input) < 11)
	{
		return 0;
	}
	
	strcpy(buf, input);

	/**
	 * This section of code was provided by voogru:
	 * http://forums.alliedmods.net/showthread.php?t=60899 
	 */

	int iServer = 0;
	int iAuthID = 0;

	char *szTmp = strtok(buf, ":");
	while((szTmp = strtok(NULL, ":")))
	{
		char *szTmp2 = strtok(NULL, ":");
		if(szTmp2)
		{
			iServer = atoi(szTmp);
			iAuthID = atoi(szTmp2);
		}
	}

    if(iAuthID == 0)
        return 0;

    uint64 i64friendID = (uint64)iAuthID * 2;
#ifdef PLATFORM_LINUX
    i64friendID += 76561197960265728LLU + iServer;
#else
	i64friendID += 76561197960265728 + iServer;
#endif

	CSteamID steamID = CSteamID(i64friendID);
#ifdef PLATFORM_LINUX
	snprintf(buf, static_cast<size_t>(params[3]), "%llu", steamID.ConvertToUint64());
#else
	snprintf(buf, static_cast<size_t>(params[3]), "%I64d", steamID.ConvertToUint64());
#endif

	return 1;
}

void SteamID::SDK_OnAllLoaded()
{
	sharesys->AddNatives(myself, MyNatives);
}
